import ts from "typescript";
import SchemaTypeBuilder from "./SchemaTypeBuilder.js";
import QueryTypeBuilder from "./QueryTypeBuilder.js";
import { replaceRange } from "../util.js";
import fs from "fs";
/**
 * The file visitor is ephemeral. Created each time we visit a file.
 */
export default class FileVisitor {
    schemaCache;
    dag;
    sourceFile;
    sqlTemplates = [];
    schemaTemplates = [];
    constructor(schemaCache, dag, sourceFile) {
        this.schemaCache = schemaCache;
        this.dag = dag;
        this.sourceFile = sourceFile;
    }
    visitAll(checker) {
        this.dag.orphan(this.sourceFile.fileName);
        this.collectAllNodes(this.sourceFile, checker);
        const schemaTypeBuilder = new SchemaTypeBuilder(this.schemaCache, this.sourceFile);
        const schemaFixes = schemaTypeBuilder.buildResidentTypes(this.schemaTemplates);
        const queryFixes = new QueryTypeBuilder(this.schemaCache, this.dag, this.sourceFile).buildQueryTypes(this.sqlTemplates, checker);
        this.applyFixes(schemaFixes.concat(queryFixes));
    }
    visitSchemaDefs(checker) {
        this.collectSchemaNodes(this.sourceFile, checker);
        const schemaFixes = new SchemaTypeBuilder(this.schemaCache, this.sourceFile).buildResidentTypes(this.schemaTemplates);
        this.applyFixes(schemaFixes);
    }
    visitQueryDefs(checker) {
        this.collectQueryNodes(this.sourceFile, checker);
        const queryFixes = new QueryTypeBuilder(this.schemaCache, this.dag, this.sourceFile).buildQueryTypes(this.sqlTemplates, checker);
        this.applyFixes(queryFixes);
    }
    applyFixes(fixes) {
        if (fixes.length == 0) {
            return;
        }
        let offset = 0;
        for (const [range, replacement] of fixes) {
            const text = this.sourceFile.getFullText();
            const updatedText = replaceRange(text, range[0], range[1], replacement);
            // if replacement is _longer_ than what was replaced then all other replacements shift further away
            // if replacement is _shorter_ then they shift closer
            offset += replacement.length - (range[1] - range[0]);
            fs.writeFileSync(this.sourceFile.fileName, updatedText);
        }
    }
    collectAllNodes(node, checker) {
        if (!ts.isTaggedTemplateExpression(node)) {
            ts.forEachChild(node, (node) => this.collectAllNodes(node, checker));
            return;
        }
        const tagName = node.tag.getText();
        if (tagName.endsWith(".sql")) {
            this.sqlTemplates.push(node);
        }
        else if (tagName.endsWith("schema")) {
            this.schemaTemplates.push(node);
        }
    }
    collectSchemaNodes(node, checker) {
        if (!ts.isTaggedTemplateExpression(node)) {
            ts.forEachChild(node, (node) => this.collectSchemaNodes(node, checker));
            return;
        }
        const tagName = node.tag.getText();
        if (tagName.endsWith("schema")) {
            this.schemaTemplates.push(node);
        }
    }
    collectQueryNodes(node, checker) {
        if (!ts.isTaggedTemplateExpression(node)) {
            ts.forEachChild(node, (node) => this.collectQueryNodes(node, checker));
            return;
        }
        const tagName = node.tag.getText();
        if (tagName.endsWith(".sql")) {
            this.sqlTemplates.push(node);
        }
    }
}
//# sourceMappingURL=FileVisitor.js.map